
local L = AceLibrary("AceLocale-2.2"):new("GridManaBars")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil
local AceOO = AceLibrary("AceOO-2.0")
local RL = AceLibrary("Roster-2.1")

local petbars = false

GridMBStatus = GridStatus:NewModule("GridMBStatus")

GridMBStatus.menuName = "ManaBar"

GridMBStatus.defaultDB = {
	debug = false,
	size = 15,
	side = "Bottom",
	hiderage = true,
	hidepetbars = true,
	color = { r = 0, g = 0.5, b = 1.0, a = 1.0 },
	ecolor = { r = 1, g = 1, b = 0, a = 1.0 },
	rcolor = { r = 1, g = 0, b = 0, a = 1.0 },
	unit_mana = {
		color = { r=1, g=1, b=1, a=1 },
		text = "ManaBar",
		enable = true,
		priority = 30,
		range = false
	}
}

GridMBStatus.options = false

local manabar_options = {
	["Manabar size"] = {
		type = "range",
		name = L["Size"],
		desc = L["Percentage of frame for mana bar"],
		max = 90,
		min = 5,
		step = 1,
		get = function ()
			return GridMBStatus.db.profile.size
			end,
		set = function(v)
			GridMBStatus.db.profile.size = v
			GridFrame:ResizeAllFrames()
			GridFrame:ScheduleEvent("GridFrame_UpdateLayoutSize", "Grid_ReloadLayout", 0.5)
		end
	},
	["Manabar side"] = {
		type = "text",
		name = L["Side"],
		desc = L["Side of frame manabar attaches to"],
		get = function ()
			return GridMBStatus.db.profile.side
			end,
		set = function(v)
			GridMBStatus.db.profile.side = v
			GridFrame:WithAllFrames(function (f) f:ReAnchor(v) end)
		end,
		validate={["Left"] = L["Left"], ["Top"] = L["Top"], ["Right"] = L["Right"], ["Bottom"] = L["Bottom"] },
	},
	["Hide Non-Mana"] = {
		type = "toggle",
		name = L["Hide Non-Mana"],
		desc = L["Hide power bars for non-mana users"],
		get = function()
			return GridMBStatus.db.profile.hiderage
		end,
		set = function(v)
			GridMBStatus.db.profile.hiderage = v
			GridFrame:WithAllFrames(function (f) f:ShowOrHideMB(v) end)
		end,
	},
	["PowerBar Colours"] = {
		type = "group",
		name = L["PowerBar Colours"],
		desc = L["Colours of the various power bars"],
		args = {
		["ManaBar color"] = {
			type = "color",
			name = L["Manabar color"],
			desc = L["Color of mana bars"],
			hasAlpha = true,
			get = function()
				local s = GridMBStatus.db.profile.color
				return s.r, s.g, s.b, s.a
			end,
			set = function(r,g,b,a)
				local s = GridMBStatus.db.profile.color
				s.r, s.g, s.b, s.a = r, g, b, a
				GridFrame:WithAllFrames(function (f) f:SetManaBarColor() end)
			end
			},
		["EnergyBar color"] = {
			type = "color",
			name = L["Energybar color"],
			desc = L["Color of energy bars"],
			hasAlpha = true,
			get = function()
				local s = GridMBStatus.db.profile.ecolor
				return s.r, s.g, s.b, s.a
			end,
			set = function(r,g,b,a)
				local s = GridMBStatus.db.profile.ecolor
				s.r, s.g, s.b, s.a = r, g, b, a
				GridFrame:WithAllFrames(function (f) f:SetManaBarColor() end)
			end
			},
		["RageBar color"] = {
			type = "color",
			name = L["Ragebar color"],
			desc = L["Color of rage bars"],
			hasAlpha = true,
			get = function()
				local s = GridMBStatus.db.profile.rcolor
				return s.r, s.g, s.b, s.a
			end,
			set = function(r,g,b,a)
				local s = GridMBStatus.db.profile.rcolor
				s.r, s.g, s.b, s.a = r, g, b, a
				GridFrame:WithAllFrames(function (f) f:SetManaBarColor() end)
			end
			},

		}
	},
	["Hide Pet power bars"] = {
		type = "toggle",
		name = L["Hide Pet-Bars"],
		desc = L["Hide power bars for pets"],
		get = function()
			return GridMBStatus.db.profile.hidepetbars
		end,
		set = function(v)
			GridMBStatus.db.profile.hidepetbars=v
		end,
	}
}
--local statusmap = GridFrame.db.profile.statusmap
local statusmap = GridFrame.defaultDB.statusmap
if not statusmap["manabar"] then
	statusmap["manabar"] = {
		unit_mana = true,
	}
end


if(GridFrame and GridFrame.db and GridFrame.db.profile) then
	statusmap = GridFrame.db.profile.statusmap
	if not statusmap["manabar"] then
		statusmap["manabar"] = {
			unit_mana = true,
		}
	end
end

-- hooking stuff, taken from GridSideIndicators

local ManaBarGridFrameClass = AceOO.Class(GridFrame.frameClass)

local _frameClass = nil
if ( not _frameClass ) then
	_frameClass = GridFrame.frameClass
	GridFrame.frameClass = ManaBarGridFrameClass
end

function ManaBarGridFrameClass.prototype:SetManaBar(value, max)
	if not self.unitName then return end
	if not self.frame.ManaBar then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:SetManaBar("..self.unitName..","..value..","..max..")")
	local ro = RL:GetUnitObjectFromName(self.unitName)
	if not ro then return end
	if max == nil then
		max = 100
	end
	if not UnitIsConnected(ro.unitid) then return end
	if GridStatusHealth.db.profile.unit_health.deadAsFullHealth and
			UnitIsDeadOrGhost(ro.unitid) then
		value=max
	end
	self.frame.ManaBar:SetValue(value/max*100)
	self:SetManaBarColor()
end

function ManaBarGridFrameClass.prototype:SetManaBarColor()
	if not self.unitName then return end
	if not self.frame.ManaBar then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:SetManaBarColor("..self.unitName..")")
	local ro = RL:GetUnitObjectFromName(self.unitName)
	if not ro then return end
	local pt = UnitPowerType(ro.unitid)
	local col = nil
	local deadcol = { r=0.5, g=0.5, b=0.5, a=1 }

	if pt==3 or ro.class == "ROGUE" then
		col = GridMBStatus.db.profile.ecolor
	elseif pt==1 or ro.class == "WARRIOR" then
		col = GridMBStatus.db.profile.rcolor
	else
		col = GridMBStatus.db.profile.color
	end

--DEFAULT_CHAT_FRAME:AddMessage("SMC() for "..tostring(self.unitName))
	if GridStatusHealth.db.profile.unit_health.deadAsFullHealth and
		UnitIsDeadOrGhost(ro.unitid) then
--DEFAULT_CHAT_FRAME:AddMessage("dead - using dead colours")

		col = deadcol
	end

	if GridFrame.db.profile.invertBarColor then
--DEFAULT_CHAT_FRAME:AddMessage("using inverted colours")
		self.frame.ManaBar:SetStatusBarColor(col.r,col.g,col.b,col.a)
		self.frame.BarMBG:SetVertexColor(0,0,0,0.8)
	else
--DEFAULT_CHAT_FRAME:AddMessage("using noninverted colours")
		self.frame.ManaBar:SetStatusBarColor(0,0,0,0.8)
		self.frame.BarMBG:SetVertexColor(col.r,col.g,col.b,col.a)
	end
end

function ManaBarGridFrameClass.prototype:InvertBarColor()
	ManaBarGridFrameClass.super.prototype.InvertBarColor(self)
--DEFAULT_CHAT_FRAME:AddMessage("FC:InvertBarColor("..self.unitName..")")
	if self.frame.ManaBar then
		self:SetManaBarColor()
	end
end

function ManaBarGridFrameClass.prototype:SetFrameTexture(texture)
	ManaBarGridFrameClass.super.prototype.SetFrameTexture(self, texture)

	if not self.frame.ManaBar then return end

	self.frame.ManaBar:SetStatusBarTexture(texture)
end

function ManaBarGridFrameClass.prototype:SetIndicator(indicator, color, text, value, maxValue, texture)
	ManaBarGridFrameClass.super.prototype.SetIndicator(self,indicator, color, text, value, maxValue, texture)
	if indicator == "manabar" then
--DEFAULT_CHAT_FRAME:AddMessage("SI("..self.unitName..")")
		if not self.frame[indicator] then
			self:CreateIndicator(indicator)
		end
		if value and maxValue then
			self:SetManaBar(value,maxValue)
		end
	end
end

function ManaBarGridFrameClass.prototype:SetWidth(width)
	if InCombatLockdown() then return end
	ManaBarGridFrameClass.super.prototype.SetWidth(self, width)

	if not self.frame.ManaBar then return end

	if self.frame.hideMB==true then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:SetWidth("..self.unitName..","..width..")")

	local f=self.frame;
	local side = GridMBStatus.db.profile.side
	local mb, hb

	if side == "Left" or side == "Right" then
		mb = (width-4) * ((GridMBStatus.db.profile.size)/100)
		hb = (width-4) - mb
	else
		mb = width-4
		hb = width-4
	end

	f.Bar:SetWidth(hb)
	f.BarBG:SetWidth(hb)

	f.ManaBar:SetWidth(mb)
	f.BarMBG:SetWidth(mb)

end

function ManaBarGridFrameClass.prototype:SetHeight(height)
	if InCombatLockdown() then return end
	ManaBarGridFrameClass.super.prototype.SetHeight(self, height)

	if not self.frame.ManaBar then return end

	if self.frame.hideMB==true then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:SetHeight("..self.unitName..","..height..")")

	local f=self.frame;
	local side = GridMBStatus.db.profile.side
	local mb, hb

	if side == "Left" or side == "Right" then
		mb = height-4
		hb = height-4
	else
		mb = (height-4) * ((GridMBStatus.db.profile.size)/100)
		hb = (height-4) - mb
	end

	f.Bar:SetHeight(hb)
	f.BarBG:SetHeight(hb)

	f.ManaBar:SetHeight(mb)
	f.BarMBG:SetHeight(mb)

end

function ManaBarGridFrameClass.prototype:CreateIndicator(indicator)
	ManaBarGridFrameClass.super.prototype.CreateIndicator(self, indicator)
	if indicator == "manabar" then
--DEFAULT_CHAT_FRAME:AddMessage("FC:CI("..self.unitName..")")
		local f = self.frame
		local texture = SML and SML:Fetch("statusbar", GridFrame.db.profile.texture) or "Interface\\Addons\\Grid\\gradient32x32"
		local side = GridMBStatus.db.profile.side

		f.BarMBG = f:CreateTexture()
		f.BarMBG:SetTexture(texture)

		f.ManaBar = CreateFrame("StatusBar", nil, f)
		f.ManaBar:SetStatusBarTexture(texture)
		f.ManaBar:SetMinMaxValues(0,100)
		f.ManaBar:SetValue(100)

		self:ReAnchor(side)
		self:SetManaBarColor()
		self:ShowOrHideMB()
	end
end

function GridMBStatus:OnInitialize()
	if GridLayout.options.args["petcolor"] then
		petbars = true
	else
		manabar_options["Hide Pet power bars"] = nil
	end
	self.super.OnInitialize(self)
	self:RegisterStatus('unit_mana',L["Mana Bar"], manabar_options, true)
	GridStatus.options.args['unit_mana'].args['color'] = nil
end

function ManaBarGridFrameClass.prototype:HideManaBar(v)
  local f = self.frame
  if not f.ManaBar then return end

--DEFAULT_CHAT_FRAME:AddMessage("FC:HideManaBar("..self.unitName..","..tostring(v)..")")
  f.hideMB = v

  h = f:GetHeight()
  w = f:GetWidth()

  if v then
	  f.ManaBar:Hide()
	  f.BarMBG:Hide()
	  f.BarMBG:SetVertexColor(0, 0, 0, 0)
  else
	f.ManaBar:Show()
	f.BarMBG:Show()
	local ro = RL:GetUnitObjectFromName(self.unitName)
	local cur, max = UnitMana(ro.unitid), UnitManaMax(ro.unitid)
	self:SetManaBar(cur,max)
  end

  self:SetHeight(h)
  self:SetWidth(w)
end

function ManaBarGridFrameClass.prototype:ShowOrHideMB()
	if not self.unitName then return end
	local ro = RL:GetUnitObjectFromName(self.unitName)
	if not ro then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:ShowOrHideMB("..self.unitName..")")

	if petbars and string.find(ro.unitid, "pet") then
			status = GridMBStatus.db.profile.hidepetbars
	elseif (ro.class == "WARRIOR" or ro.class == "ROGUE") then
		status = GridMBStatus.db.profile.hiderage
	elseif (ro.class == "DRUID") then
		local pt = UnitPowerType(ro.unitid)
		if pt==1 or pt==3 then
			status = GridMBStatus.db.profile.hiderage
		else
			status = false
		end
	else
		status = false
	end

	self:HideManaBar(status)
end

function ManaBarGridFrameClass.prototype:ReAnchor(side)

	local f = self.frame
	if not f.ManaBar then return end
--DEFAULT_CHAT_FRAME:AddMessage("FC:ReAnchor("..self.unitName..","..side..")")

	f.Bar:ClearAllPoints()
	f.ManaBar:ClearAllPoints()
	f.BarBG:ClearAllPoints()
	f.BarMBG:ClearAllPoints()

	if side == "Right" then
		f.Bar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.BarBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.ManaBar:SetPoint("TOPRIGHT", f, "TOPRIGHT",-2,-2)
		f.BarMBG:SetPoint("TOPRIGHT", f, "TOPRIGHT",-2,-2)
		f.ManaBar:SetOrientation("VERTICAL")
	elseif side == "Left" then
		f.ManaBar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.BarMBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.Bar:SetPoint("TOPRIGHT", f, "TOPRIGHT",-2,-2)
		f.BarBG:SetPoint("TOPRIGHT", f, "TOPRIGHT",-2,-2)
		f.ManaBar:SetOrientation("VERTICAL")
	elseif side == "Bottom" then
		f.Bar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.BarBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.ManaBar:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT",2,2)
		f.BarMBG:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT",2,2)
		f.ManaBar:SetOrientation("HORIZONTAL")
	else
		f.ManaBar:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.BarMBG:SetPoint("TOPLEFT", f, "TOPLEFT",2,-2)
		f.Bar:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT",2,2)
		f.BarBG:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT",2,2)
		f.ManaBar:SetOrientation("HORIZONTAL")
	end

	self:SetWidth(f:GetWidth())
	self:SetHeight(f:GetHeight())
end

function GridMBStatus:OnEnable()
--	self:RegisterBucketEvent("UNIT_MANA", 0.2)
--	self:RegisterBucketEvent("UNIT_RAGE", 0.2)
--	self:RegisterBucketEvent("UNIT_ENERGY", 0.2)
	self:RegisterBucketEvent("PARTY_MEMBERS_CHANGED", 0.2)
	self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Grid_UnitDeath")
	self:ScheduleRepeatingEvent(self.manabar_update, 2, self)
	self:RegisterEvent("UNIT_MANA","UpdateUnit")
	self:RegisterEvent("UNIT_RAGE","UpdateUnit")
	self:RegisterEvent("UNIT_ENERGY","UpdateUnit")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
end

function GridMBStatus:manabar_update()
	local side = GridMBStatus.db.profile.side
	for k,v in pairs(GridFrame.registeredFrames) do
		if v.unitName then
			local ro = RL:GetUnitObjectFromName(v.unitName)
			if ro then
				local cur, max = UnitMana(ro.unitid), UnitManaMax(ro.unitid)
				v:ShowOrHideMB()
				v:SetManaBar(cur,max)
			end
		end
	end
end

function GridMBStatus:PARTY_MEMBERS_CHANGED()
	local side = GridMBStatus.db.profile.side
	for k,v in pairs(GridFrame.registeredFrames) do
		if v.unitName then
 			v:ReAnchor(side)
			v:SetManaBarColor()
			v:ShowOrHideMB()
		end
	end
end

function GridMBStatus:UNIT_DISPLAYPOWER(unitid)
--DEFAULT_CHAT_FRAME:AddMessage("MB:UNIT_DISPLAY_POWER("..unitid..")")
	local lC,eC = UnitClass(unitid)
	if eC == "DRUID" then
		local nN,rN = UnitName(unitid)
		for k,v in pairs(GridFrame.registeredFrames) do
			if v.unitName == nN then
				v:SetManaBarColor()
				v:ShowOrHideMB()
			end
		end
	end
end

--function GridMBStatus:UNIT_RAGE(units)
--	for unitid in pairs(units) do
--		self:UpdateUnit(unitid)
--	end
--end
--
--function GridMBStatus:UNIT_ENERGY(units)
--	for unitid in pairs(units) do
--		self:UpdateUnit(unitid)
--	end
--end
--
--function GridMBStatus:UNIT_MANA(units)
--	for unitid in pairs(units) do
--		self:UpdateUnit(unitid)
--	end
--end

function GridMBStatus:Grid_UnitJoined(name,unitid)
--DEFAULT_CHAT_FRAME:AddMessage("Grid_UnitJoined : "..tostring(unitid)..","..tostring(unitid))
	for k,v in pairs(GridFrame.registeredFrames) do
		if v.unitName == name then
			v.unitid = unitid
			v:SetManaBarColor()
			v:ShowOrHideMB()
			self:UpdateUnit(unitid)
		end
	end
end

function GridMBStatus:Grid_UnitDeath(name)
--DEFAULT_CHAT_FRAME:AddMessage("Grid_UnitDeath("..name..")")
	for k,v in pairs(GridFrame.registeredFrames) do
		if v.unitName == name then
			v:SetManaBarColor()
			v:ShowOrHideMB()
		end
	end
end

function GridMBStatus:UpdateUnit(unitid)

	local name = UnitName(unitid)
	if not name then return end
	local _,c = UnitClass(unitid)
	if string.find(unitid, "pet") and not petbars then return end

	local cur, max = UnitMana(unitid), UnitManaMax(unitid)
	local priority = 70

	if cur==max then
		priority=1
	end

--DEFAULT_CHAT_FRAME:AddMessage("UU("..unitid..")")

	self.core:SendStatusGained(name, "unit_mana", priority, nil,
		nil,nil,cur,max,nil)
end

