﻿-- ----------------------------------------------------------------------------
-- GridIndicatorText3 by kunda
-- ----------------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridIndicatorText3")
local AceOO = AceLibrary("AceOO-2.0")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil

GridIndicatorText3 = GridStatus:NewModule("GridIndicatorText3")
GridIndicatorText3.defaultDB = {
	Text3fontSize = 8,
	Text3font = "Friz Quadrata TT",
	}

local indicators = GridFrame.frameClass.prototype.indicators
table.insert(indicators, { type = "text3", order = 5.5, name = L["Center Text 3"] })

local statusmap = GridFrame.db.profile.statusmap
if not statusmap["text3"] then
	statusmap["text3"] = {}
end

local GridIndicatorText3FrameClass = AceOO.Class(GridFrame.frameClass)

local _frameClass = nil
if not _frameClass then
	_frameClass = GridFrame.frameClass
	GridFrame.frameClass = GridIndicatorText3FrameClass
end

function GridIndicatorText3:OnEnable()
	local opt = GridFrame.options.args.advanced.args
	opt["text3"] = {
		type = "group",
		name = L["Center Text 3"],
		desc = L["Center Text 3 options."],
		order = 110,
		args = {
			["text3fontsize"] = {
				type = "range",
				name = L["Center Text 3 Font Size"],
				desc = L["Adjust the font size for Center Text 3."],
				order = 111,
				min = 6,
				max = 24,
				step = 1,
				get = function ()
					return GridIndicatorText3.db.profile.Text3fontSize
				end,
				set = function (v)
					GridIndicatorText3.db.profile.Text3fontSize = v
					font = SML and SML:Fetch('font', GridIndicatorText3.db.profile.Text3font) or STANDARD_TEXT_FONT
					GridFrame:WithAllFrames(function (f) f:SetFrameFont(font, v, "text3") end)
				end,
			}
		}
	}
	if SML then
		opt["text3"].args["text3font"] = {
			type = "text",
			name = L["Center Text 3 Font"],
			desc = L["Adjust the font setting for Center Text 3."],
			order = 112,
			validate = SML:List("font"),
			get = function ()
				return GridIndicatorText3.db.profile.Text3font
			end,
			set = function (v)
				GridIndicatorText3.db.profile.Text3font = v
				local font = SML:Fetch("font", v)
				local fontsize = GridIndicatorText3.db.profile.Text3fontSize
				GridFrame:WithAllFrames(function (f) f:SetFrameFont(font, fontsize, "text3") end)
			end,
		}
	end
end

function GridIndicatorText3FrameClass.prototype:CreateIndicator(indicator)
	GridIndicatorText3FrameClass.super.prototype.CreateIndicator(self, indicator)
	local f = self.frame
	if indicator == "text3" then
		local font = SML and SML:Fetch("font", GridIndicatorText3.db.profile.Text3font) or STANDARD_TEXT_FONT
		f.Text3 = f.Bar:CreateFontString(nil, "ARTWORK")
		f.Text3:SetFontObject(GameFontHighlightSmall)
		f.Text3:SetFont(font, GridIndicatorText3.db.profile.Text3fontSize)
		f.Text3:SetJustifyH("CENTER")
		f.Text3:SetJustifyV("CENTER")
		f.Text3:SetPoint("CENTER", f, "CENTER")
		f.Text3:Hide()
	end
end

function GridIndicatorText3FrameClass.prototype:SetIndicator(indicator, color, text, value, maxValue, texture)
	GridIndicatorText3FrameClass.super.prototype.SetIndicator(self, indicator, color, text, value, maxValue, texture)
	local f = self.frame
	if indicator == "text3"	then
		if not f.Text3 then
			self:CreateIndicator(indicator)
		end
		local text = text:utf8sub(1, GridFrame.db.profile.textlength)
		f.Text3:SetText(text)
		if text and text ~= "" then
			f.Text3:Show()
		else
			f.Text3:Hide()
		end
		if color then
			f.Text3:SetTextColor(color.r, color.g, color.b, color.a)
		end
	end
end

function GridIndicatorText3FrameClass.prototype:ClearIndicator(indicator)
	GridIndicatorText3FrameClass.super.prototype.ClearIndicator(self, indicator)
	local f = self.frame
	if indicator == "text3"	then
		if f.Text3 then
			f.Text3:Hide()
		end
	end
end

function GridIndicatorText3FrameClass.prototype:SetFrameFont(font, size, check)
	if check then
		GridIndicatorText3FrameClass.super.prototype.SetFrameFont(self, GridFrame.db.profile.font, GridFrame.db.profile.fontSize)
		local f = self.frame
		if f.Text3 then
			f.Text3:SetFont(font, size)
		end
	else
		GridIndicatorText3FrameClass.super.prototype.SetFrameFont(self, font, size)
	end
end