----------------------------------------------------------------
--	Locals
----------------------------------------------------------------

local Aura = AceLibrary("SpecialEvents-Aura-2.0")

local L = setmetatable(GRIDSTATUSHEALINGREDUCED_LOCALS or {}, {
		__index = function(self, key)
			self[key] = key
			return key
		end
	})
if GRIDSTATUSHEALINGREDUCED_LOCALS then
	GRIDSTATUSHEALINGREDUCED_LOCALS = nil
end

----------------------------------------------------------------
--	Debuffs
----------------------------------------------------------------

local healingReductionBuffs = {
	[L["Dampen Magic"]] = true,					-- 500, mages (lower ranks have smaller reduction)
}

local healingReductionDebuffs = {
	[L["Aimed Shot"]] = true,					-- 50% - hunters
	[L["Arcing Smash"]] = true,					-- 50% - Gurtogg Bloodboil (Black Temple)
	[L["Blood Fury"]] = true,					-- 50% - orcs
	[L["Brood Affliction: Green"]] = true,		-- 50% - Chromaggus (Blackwing Lair)
	[L["Curse of the Bleeding Hollow"]] = true,	-- 15% - Bleeding Hollow orcs (Hellfire Peninsula)
	[L["Curse of the Deadwood"]] = true,		-- 50% - Deadwood furbolgs (Felwood)
	[L["Dark Touched"]] = true,					--  4% x 25 - Grand Warlock Alythess (Sunwell Plateau)
	[L["Deathblow"]] = true,					-- 50% - ???
	[L["Demolish"]] = true,						-- 75% - Negatron (Netherstorm)
	[L["Ebon Poison"]] = true,					-- 25% - Blackfang Tarantulas (The Black Morass)
	[L["Filet"]] = true,						-- 50% - Spectral Chefs (Karazhan)
	[L["Gehennas' Curse"]] = true,				-- 75% - Gehennas (Molten Core)
	[L["Hex of Weakness"]] = true,				-- 20% - troll priests
	[L["Magma-Thrower's Curse"]] = true,		-- 50% - Sulfuron Magma-Thrower (The Arcatraz)
	[L["Mortal Cleave"]] = true,				-- 50% - High Priestess Thekal (Zul'Gurub)
	[L["Mortal Strike"]] = true,				-- 50% - warriors
	[L["Mortal Wound"]] = true,					-- 10% x 7 - Temporus (The Black Morass)
	[L["Necrotic Poison"]] = true,				-- 90% - Maexxna (Naxxramas)
	[L["Nether Portal - Dominance"]] = true,	--  1% x 99 - Netherspite (Karazhan)
	[L["Solar Strike"]] = true,					-- 50% - ??? (The Mechanar)
	[L["Soul Strike"]] = true,					-- 50% - Ethereal Crypt Raiders (Mana-Tombs)
	[L["Touch of the Forgotten"]] = true,		-- 690 - ??? (Auchenai Crypts)
	[L["Veil of Shadow"]] = true,				-- 75% - Nefarian (Blackwing Lair)
	[L["Wound Poison"]] = true,					-- 10% x 5 - rogues
}

local healingPreventionDebuffs = {
	[L["Aura of Suffering"]] = true,			-- Essence of Suffering (Black Temple)
	[L["Enfeeble"]] = true,						-- Prince Malchezaar (Karazhan)
}

----------------------------------------------------------------
--	Module
----------------------------------------------------------------

GridStatusHealingReduced = GridStatus:NewModule("GridStatusHealingReduced")

local mod = GridStatusHealingReduced
mod.menuName = L["Healing Reduced"]
mod.options = false
mod.defaultDB = {
	debug = false,
	alert_healingReduced = {
		text = L["Healing Reduced"],
		enable = true,
		color = { r = 0.8, g = 0.4, b = 0.8, a = 1 },
		priority = 90,
		range = true,
	},
	alert_healingPrevented = {
		text = L["Healing Prevented"],
		enable = true,
		color = { r = 0.8, g = 0.4, b = 0.8, a = 1 },
		priority = 99,
		range = true,
	},
}

function mod:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatus("alert_healingReduced", L["Healing Reduced"], nil, true)
	self:RegisterStatus("alert_healingPrevented", L["Healing Prevented"], nil, true)
end

function mod:OnEnable()
	self:RegisterEvent("SpecialEvents_UnitDebuffGained", "UnitAuraChanged")
	self:RegisterEvent("SpecialEvents_UnitDebuffLost", "UnitAuraChanged")
end

local i, settings
function mod:UnitAuraChanged(u)
	i = 0
	for k in pairs(healingPreventionDebuffs) do
		if Aura:UnitHasDebuff(u, k) then
			i = i + 1
		end
	end
	if i > 0 then
		settings = self.db.profile.alert_healingPrevented
		self.core:SendStatusGained(UnitName(u), "alert_healingPrevented", settings.priority, (settings.range and 40), settings.color, settings.text)
		return
	else
		self.core:SendStatusLost(UnitName(u), "alert_healingPrevented")
	end

	i = 0
	for k in pairs(healingReductionDebuffs) do
		if Aura:UnitHasDebuff(u, k) then
			i = i + 1
		end
	end
	if i > 0 then
		settings = self.db.profile.alert_healingReduced
		self.core:SendStatusGained(UnitName(u), "alert_healingReduced", settings.priority, (settings.range and 40), settings.color, settings.text)
	else
		self.core:SendStatusLost(UnitName(u), "alert_healingReduced")
	end
end