﻿-- ----------------------------------------------------------------------------
-- GridStatusTargetName by kunda
-- ----------------------------------------------------------------------------
local RL = AceLibrary("Roster-2.1")
local L = AceLibrary("AceLocale-2.2"):new("GridStatusTargetName")

GridStatusTargetName = GridStatus:NewModule("GridStatusTargetName")
GridStatusTargetName.menuName = L["Target Name"]

GridStatusTargetName.defaultDB = {
	debug = false,
	alert_targetname = {
		text =  L["Target Name"],
		enable = true,
		color = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
		priority = 10,
		range = false,
	}
}

GridStatusTargetName.options = false

function GridStatusTargetName:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("alert_targetname", L["Target Name"], nil, true)
end

function GridStatusTargetName:OnEnable()
	self:RegisterEvent("UNIT_TARGET")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
end

function GridStatusTargetName:PARTY_MEMBERS_CHANGED()
	if not self.db.profile.alert_targetname.enable then return end
	for u in RL:IterateRoster(true) do
		self:UNIT_TARGET(u.unitid)
	end
end

function GridStatusTargetName:UNIT_TARGET(e_unitid)
	local settings = self.db.profile.alert_targetname
	if not settings.enable then return end
	local unitname = UnitName(e_unitid)

	if UnitExists(e_unitid.."target") then
		local targetname = UnitName(e_unitid.."target")
		for u in RL:IterateRoster(true) do
			if targetname == UnitName(u.unitid) then
				local color = GridStatus:UnitColor(u)
				self.core:SendStatusGained(unitname, "alert_targetname",
					settings.priority,
					(settings.range and 40),
					{ r = color.r, g = color.g, b = color.b, a = settings.color.a },
					"T:"..targetname)
				break
			else
				self.core:SendStatusLost(unitname, "alert_targetname")
			end
		end
	else
		self.core:SendStatusLost(unitname, "alert_targetname")
	end
end